==> VVAS-DataCenter-core-6.0.0-sources.jar: com/viontech/keliu/processor/VionMessageProcessor.java <==
package com.viontech.keliu.processor;



import com.viontech.keliu.model.Body;
import com.viontech.keliu.model.Head;
import com.viontech.keliu.model.Message;
import io.netty.buffer.ByteBuf;
import io.netty.buffer.ByteBufAllocator;

import java.nio.charset.Charset;
import java.nio.charset.UnsupportedCharsetException;
import java.util.Arrays;


public abstract class VionMessageProcessor {
	
	public static final int MESSAGE_TYPE_BINARY = 2;
	public static final int MESSAGE_TYPE_XML = 1;
	
	
	protected Charset charset = null;
	public VionMessageProcessor() {
		if(!Charset.isSupported("GB2312")){
			throw new UnsupportedCharsetException("GB2312");
		}
		charset = Charset.forName("GB2312");
	}

	public Message process(ByteBuf byteBuf) {
		return process(byteBuf,null);
	}
	public Message process(ByteBuf byteBuf,String deviceSerialnum){
		Message message = new Message();
		if(deviceSerialnum != null && message.getSerialNum() == null){
			message.setSerialNum(deviceSerialnum);
		}
		Head head = parseHead(byteBuf);
		message.setHead(head);
		Body body = parseBody(byteBuf);
		message.setBody(body);
		int readable = byteBuf.readerIndex()+byteBuf.readableBytes();
		byte[] bytes = new byte[readable];
		byteBuf.getBytes(0, bytes);
		boolean success = doProcess(message,bytes);
		return message;
		
	}
	
	public abstract boolean doProcess(Message message,byte[] bytes);

	private Head parseHead(ByteBuf byteBuf){
		try {
			Head head = new Head();
			long size = byteBuf.readUnsignedInt();
			head.setSize(size);
			long check = byteBuf.readUnsignedInt();
			head.setCheck(check);
			long version = byteBuf.readUnsignedInt();
			head.setVersion(version);
			long type = byteBuf.readUnsignedInt();
			head.setType(type);
			return head;
		} catch (Exception e) {
			e.printStackTrace();
		}
		return null;
	}

	
	
	public abstract Body parseBody(ByteBuf byteBuf);

	public abstract boolean isValid(ByteBuf byteBuf);
	
	public abstract void log(Message message);

	
	private boolean buildByteBufHead(Head head,ByteBuf byteBuf){
		try {
			byteBuf.writeInt(0);
			byteBuf.writeInt(0);
			byteBuf.writeInt((int) head.getVersion());
			byteBuf.writeInt((int) head.getType());
			return true;
		} catch (Exception e) {
			e.printStackTrace();
		}
		return false;
		
	}
	
	protected abstract boolean buildByteBufBody(Body body,ByteBuf byteBuf);

	public ByteBuf buildByteBuf(Message message){
	
		if(!message.isNeedResponse()){
			return null;
		}
		ByteBuf byteBuf = ByteBufAllocator.DEFAULT.buffer();
		boolean success = buildByteBufHead(message.getHead(), byteBuf);
		if(!success){
			return null;
		}
		success = buildByteBufBody(message.getBody(), byteBuf);
		if(!success){
			return null;
		}
		int size = byteBuf.writerIndex();
		byteBuf.setInt(0, size);
		byteBuf.setInt(4, size^2005002005);
		return byteBuf;
	}
	
	protected abstract String supportCommand();
	
	protected abstract long supportVersion();
	
	
	protected byte[] transform(String str,int length){
		byte[] bytes = str.getBytes(charset);
		return Arrays.copyOf(bytes, length);
	}
	
	protected abstract String getResponseCategory();
}

==> VVAS-DataCenter-core-6.1.4-sources.jar: com/viontech/keliu/processor/VionMessageProcessor.java <==
package com.viontech.keliu.processor;


import com.viontech.keliu.content.Body;
import com.viontech.keliu.content.Head;
import com.viontech.keliu.content.Message;
import io.netty.buffer.ByteBuf;
import io.netty.buffer.ByteBufAllocator;
import org.dom4j.Document;
import org.dom4j.DocumentHelper;
import org.dom4j.Element;

import java.nio.charset.Charset;
import java.nio.charset.UnsupportedCharsetException;
import java.util.Arrays;


public abstract class VionMessageProcessor {
	protected static final String ELEMENT_NAME_ROOT = "rvss";
	protected static final String ELEMENT_NAME_COMMAND = "command";
	public static final int MESSAGE_TYPE_BINARY = 2;
	public static final int MESSAGE_TYPE_XML = 1;
	
	
	protected  static Charset charset = null;
	static{
		if(!Charset.isSupported("GB2312")){
			throw new UnsupportedCharsetException("GB2312");
		}
		charset = Charset.forName("GB2312");
	}
	public VionMessageProcessor() {

	}

	public Message process(ByteBuf byteBuf) {
		return process(byteBuf,null);
	}
	public Message process(ByteBuf byteBuf,String deviceSerialnum){
		Message message = new Message();
		if(deviceSerialnum != null && message.getSerialNum() == null){
			message.setSerialNum(deviceSerialnum);
		}
		Head head = parseHead(byteBuf);
		message.setHead(head);
		Body body = parseBody(byteBuf);
		message.setBody(body);
		int readable = byteBuf.readerIndex()+byteBuf.readableBytes();
		byte[] bytes = new byte[readable];
		byteBuf.getBytes(0, bytes);
		boolean success = doProcess(message,bytes);
		return message;
		
	}
	
	public abstract boolean doProcess(Message message,byte[] bytes);

	protected Head parseHead(ByteBuf byteBuf){
		try {
			Head head = new Head();
			long size = byteBuf.readUnsignedInt();
			head.setSize(size);
			long check = byteBuf.readUnsignedInt();
			head.setCheck(check);
			long version = byteBuf.readUnsignedInt();
			head.setVersion(version);
			long type = byteBuf.readUnsignedInt();
			head.setType(type);
			return head;
		} catch (Exception e) {
			e.printStackTrace();
		}
		return null;
	}

	
	
	public abstract Body parseBody(ByteBuf byteBuf);

	public abstract boolean isValid(ByteBuf byteBuf);
	
	public abstract void log(Message message);

	
	private boolean buildByteBufHead(Head head,ByteBuf byteBuf){
		try {
			byteBuf.writeInt(0);
			byteBuf.writeInt(0);
			byteBuf.writeInt((int) head.getVersion());
			byteBuf.writeInt((int) head.getType());
			return true;
		} catch (Exception e) {
			e.printStackTrace();
		}
		return false;
		
	}
	
	protected abstract boolean buildByteBufBody(Body body,ByteBuf byteBuf);

	public ByteBuf buildByteBuf(Message message){
	
		if(!message.isNeedResponse()){
			return null;
		}
		ByteBuf byteBuf = ByteBufAllocator.DEFAULT.buffer();
		boolean success = buildByteBufHead(message.getHead(), byteBuf);
		if(!success){
			return null;
		}
		success = buildByteBufBody(message.getBody(), byteBuf);
		if(!success){
			return null;
		}
		int size = byteBuf.writerIndex();
		byteBuf.setInt(0, size);
		byteBuf.setInt(4, size^2005002005);
		return byteBuf;
	}
	
	public abstract String supportCommand();
	
	public abstract long supportVersion();

	public abstract long supportType();
	
	
	protected byte[] transform(String str,int length){
		byte[] bytes = str.getBytes(charset);
		return Arrays.copyOf(bytes, length);
	}
	
	protected abstract String getResponseCategory();

	public static String getCommandVersion(ByteBuf byteBuf){
		long size = byteBuf.getUnsignedInt(0);
		long version = byteBuf.getUnsignedInt(8);
		long type = byteBuf.getUnsignedInt(12);
		if(type == MESSAGE_TYPE_XML){
			Document doc = null;
			try {

				String xml = byteBuf.getCharSequence(16, (int)(size-16), charset).toString().trim();
				doc = DocumentHelper.parseText(xml); // 将字符串转为XML
				Element rootElt = doc.getRootElement(); // 获取根节点
				Element commandElement = rootElt.element(ELEMENT_NAME_COMMAND);
				String command = commandElement.getTextTrim();
				return command+"_"+type+"_"+version;
			}catch(Exception e){
				e.printStackTrace();
			}
		}else if(type == MESSAGE_TYPE_BINARY){
			CharSequence command = byteBuf.getCharSequence(80, 32, charset);
			String commandStr = command.toString().trim();
			return commandStr+"_"+type+"_"+version;
		}
		return null;
	}
}
